using FitConnect;
using FitConnect.Encryption;
using FitConnect.Models;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;

namespace ConsoleAppExample;

public static class SenderDemo {
    public static async Task Run(IConfigurationRoot config, ILogger logger) {
        var clientId = config["FitConnect:Sender:ClientId"];
        var clientSecret = config["FitConnect:Sender:ClientSecret"];
        var destinationId = config["FitConnect:Sender:DestinationId"];
        var leikaKey = config["FitConnect:Sender:LeikaKey"];


        OutputHelper.PrintSender();

        var createdSubmission = SubmissionBuilder
            .WithDestination(destinationId)
            .WithServiceType("FIT Connect Demo", leikaKey)
            .WithJsonData("{\"message\":\"Hello World\"}")
            .WithAttachments(new Attachment("./Attachments/Test.pdf", "Test Attachment"),
                new Attachment("./Attachments/Test.pdf", "Test Attachment #2"))
            .Build();

        await Client.GetSender(FitConnectEnvironment.Testing, clientId, clientSecret, logger)
            .SendAsync(createdSubmission);
    }

    public static async Task RunEncrypted(IConfigurationRoot config, ILogger logger) {
        var clientId = config["FitConnect:Sender:ClientId"];
        var clientSecret = config["FitConnect:Sender:ClientSecret"];
        var destinationId = config["FitConnect:Sender:DestinationId"];
        var leikaKey = config["FitConnect:Sender:LeikaKey"];


        var encryption = new FitEncryption("", "", null);
        Dictionary<string, string>? encryptedAttachments = null;
        string? encryptedMetadata = null;
        string? encryptedData = null;

        var publicKey = await Client
            .GetSender(FitConnectEnvironment.Testing, clientId, clientSecret, logger)
            .GetPublicKeyForDestinationAsync(destinationId);


        var encryptedSubmission = EncryptedSubmissionBuilder
            .WithDestination(destinationId)
            .WithServiceType("FIT Connect Demo", leikaKey)
            .WithEncryptedMetaData(encryptedMetadata!)
            .WithEncryptedData(encryptedData!)
            .WithEncryptedAttachments(encryptedAttachments!)
            .Build();
        var sender = await Client
            .GetSender(FitConnectEnvironment.Testing, clientId, clientSecret, logger)
            .SendAsync(encryptedSubmission);
    }
}
